@@ -2,17 +2,14 @@ package Dist::Zilla::Plugin::NextVersion::Semantic;
 BEGIN {
   $Dist::Zilla::Plugin::NextVersion::Semantic::AUTHORITY = 'cpan:YANICK';
 }
-{
-  $Dist::Zilla::Plugin::NextVersion::Semantic::VERSION = '0.1.3';
-}
 # ABSTRACT: update the next version, semantic-wise
-
+$Dist::Zilla::Plugin::NextVersion::Semantic::VERSION = '0.2.0';
 use strict;
 use warnings;
 
 use CPAN::Changes 0.20;
 use Perl::Version;
-use List::MoreUtils qw/ any /;
+use List::AllUtils qw/ any min /;
 
 use Moose;
 
@@ -23,6 +20,7 @@ with qw/
     Dist::Zilla::Role::BeforeRelease
     Dist::Zilla::Role::AfterRelease
     Dist::Zilla::Role::VersionProvider
+    Dist::Zilla::Plugin::NextVersion::Semantic::Incrementer
 /;
 
 use Moose::Util::TypeConstraints;
@@ -43,6 +41,12 @@ has change_file  => ( is => 'ro', isa=>'Str', default => 'Changes' );
 has numify_version => ( is => 'ro', isa => 'Bool', default => 0 );
 
 
+has format => (
+    is => 'ro',
+    isa => 'Str',
+    default => '%d.%3d.%3d',
+);
+
 
 has major => (
     is => 'ro',
@@ -58,7 +62,7 @@ has minor => (
     is => 'ro',
     isa => 'ChangeCategory',
     coerce => 1,
-    default => sub { [ 'ENHANCEMENTS' ] },
+    default => sub { [ 'ENHANCEMENTS', 'UNGROUPED' ] },
     traits  => ['Array'],
     handles => { minor_groups => 'elements' },
 );
@@ -106,7 +110,7 @@ sub after_release {
 
   my ( $next ) = reverse $changes->releases;
 
-  $next->add_group( $self->all_groups );
+  $next->add_group( grep { $_ ne 'UNGROUPED' } $self->all_groups );
 
   $self->log_debug([ 'updating contents of %s on disk', $filename ]);
 
@@ -164,52 +168,40 @@ sub provide_version {
 sub next_version {
     my( $self, $last_version ) = @_;
 
-    my ($changes_file) = grep { $_->name eq $self->change_file } @{ $self->zilla->files };
 
-    my $changes = CPAN::Changes->load_string( $changes_file->content,
-        next_token => qr/{{\$NEXT}}/ );
+    my $new_ver = $self->increment_version( $self->increment_level );
 
-    my ($next) = reverse $changes->releases;
-
-    my $new_ver = $self->inc_version(
-        $last_version,
-        grep { scalar @{ $next->changes($_) } } $next->groups
-    );
-
-    $new_ver = $new_ver->numify if $self->numify_version;
+    $new_ver = Perl::Version->new( $new_ver )->numify if $self->numify_version;
 
     $self->log("Bumping version from $last_version to $new_ver");
     return $new_ver;
 }
 
-sub inc_version {
-    my ( $self, $last_version, @groups ) = @_;
+sub increment_level {
+    my ( $self ) = @_;
 
-    $last_version = Perl::Version->new( $last_version );
+    my ($changes_file) = grep { $_->name eq $self->change_file } @{ $self->zilla->files }
+        or die "no changelog file found\n";
 
-    for my $group ( $self->major_groups ) {
-        next unless any { $group eq $_ } @groups;
+    my $changes = CPAN::Changes->load_string( $changes_file->content,
+        next_token => qr/{{\$NEXT}}/ );
 
-        $self->log_debug( "$group change detected, major increase" );
+    my ($changelog) = reverse $changes->releases;
 
-        $last_version->inc_revision;
-        return $last_version
-    }
+    my %category_map = (
+        map( { $_ => 0 } $self->major_groups ),
+        map( { $_ => 1 } $self->minor_groups ),
+    );
 
-    for my $group ( '', $self->minor_groups ) {
-        next unless any { $group eq $_ } @groups;
+    $category_map{''} = $category_map{UNGROUPED};
 
-        my $section = $group || 'general';
-        $self->log_debug( "$section change detected, minor increase" );
+    no warnings;
 
-        $last_version->inc_version;
-        return $last_version
-    }
+    my $increment_level = min map { $category_map{$_} }
+        grep { scalar @{ $changelog->changes($_) } }  # only groups with items
+        $changelog->groups;
 
-    $self->log_debug( "revision increase" );
-
-    $last_version->inc_subversion;
-    return $last_version;
+    return (qw/MAJOR MINOR PATCH/)[$increment_level//2];
 }
 
 sub munge_files {
@@ -233,19 +225,94 @@ sub munge_files {
 
 __PACKAGE__->meta->make_immutable;
 no Moose;
+
+{
+    package Dist::Zilla::Plugin::NextVersion::Semantic::Incrementer;
+BEGIN {
+  $Dist::Zilla::Plugin::NextVersion::Semantic::Incrementer::AUTHORITY = 'cpan:YANICK';
+}
+$Dist::Zilla::Plugin::NextVersion::Semantic::Incrementer::VERSION = '0.2.0';
+use List::AllUtils qw/ first_index any /;
+
+    use Moose::Role;
+
+    requires 'previous_version', 'format';
+
+    sub nbr_version_levels {
+        my @tokens = $_[0]->format =~ /(%\d*d)/g;
+        return scalar @tokens;
+    }
+
+    sub version_lenghts {
+        return $_[0]->format =~ /%0*(\d*)/g;
+    }
+
+    sub increment_version {
+        my( $self, $level ) = @_;
+        $level ||= 'PATCH';
+
+        my @version = (0,0,0);
+
+        my $previous = $self->previous_version;
+
+        # initial version is special
+        unless ( $previous eq '0' ) {
+            my $regex = quotemeta $self->format;
+            $regex =~ s/\\%0(\d+)d/(\\d{$1})/g;
+            $regex =~ s/\\%(\d+)d/(\\d{1,$1})/g;
+            $regex =~ s/\\%d/(\\d+)/g;
+
+            @version = $previous =~ /$regex/
+                or die "previous version '$previous' doesn't match format '$self->format'" ;
+        }
+
+        my @levels = qw/ MAJOR MINOR PATCH /;
+        my $index = first_index { $level eq $_ } @levels;
+
+        $version[$index]++;
+        $version[$_] = 0 for $index+1..2;
+
+        # if the incremental level is below the number of levels we 
+        # have, increment the lowest level we consider
+        if( any { $version[$_] > 0 } $self->nbr_version_levels..2 ) {
+            $version[ $self->nbr_version_levels -1 ]++;
+            $version[$_] = 0 for $self->nbr_version_levels..2;
+        }
+
+        # exceeding sizes?
+        my @sizes = $self->version_lenghts;
+        for my $i ( grep { $sizes[$_] } reverse 0..2 ) {
+            if ( length( $version[$i] ) > $sizes[$i] ) {
+                $version[$i-1]++;
+                $version[$i] = 0;
+            }
+        }
+
+        my $version = sprintf $self->format, @version;
+        $version =~ y/ //d;
+
+        return $version;
+    }
+
+
+}
+
+
 1;
 
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dist::Zilla::Plugin::NextVersion::Semantic - update the next version, semantic-wise
 
 =head1 VERSION
 
-version 0.1.3
+version 0.2.0
 
 =head1 SYNOPSIS
 
@@ -307,6 +374,17 @@ as-if as the next version.
 For this plugin to work, your L<Dist::Zilla> configuration must also contain a plugin
 consuming the L<Dist::Zilla::Role::YANICK::PreviousVersionProvider> role.
 
+In the different configuration attributes where change group names are given,
+the special group name C<UNGROUPED> can be given to 
+specify the nameless group.
+
+    0.1.3 2013-07-18
+
+    - this item will be part of UNGROUPED.
+
+    [BUG FIXES]
+    - this one won't.
+
 =head1 PARAMETERS
 
 =head2 change_file
@@ -318,6 +396,32 @@ File name of the changelog. Defaults to C<Changes>.
 If B<true>, the version will be a number using the I<x.yyyzzz> convention instead
 of I<x.y.z>.  Defaults to B<false>.
 
+=head2 format
+
+Specifies the version format to use. Follows the '%d' convention of
+C<sprintf> (see examples below), excepts for one detail: '%3d' won't pad 
+with whitespaces, but will only determine the maximal size of the number. 
+If a version component exceeds its given
+size, the next version level will be incremented.
+
+Examples:
+
+    %d.%3d.%3d 
+        PATCH LEVEL INCREASES: 0.0.998 -> 0.0.999 -> 0.1.0
+        MINOR LEVEL INCREASES: 0.0.8 -> 0.1.0 -> 0.2.0
+        MAJOR LEVEL INCREASES: 0.1.8 -> 1.0.0 -> 2.0.0
+
+    %d.%02d%02d
+        PATCH LEVEL INCREASES: 0.0098 -> 0.00099 -> 0.0100
+        MINOR LEVEL INCREASES: 0.0008 -> 0.0100 -> 0.0200
+        MAJOR LEVEL INCREASES: 0.0108 -> 1.0000 -> 2.0000
+
+    %d.%05d
+        MINOR LEVEL INCREASES: 0.99998 -> 0.99999 -> 1.00000
+        MAJOR LEVEL INCREASES: 0.00108 -> 1.00000 -> 2.00000
+
+Defaults to '%d.%3d.%3d'.
+
 =head2 major
 
 Comma-delimited list of categories of changes considered major.
@@ -326,7 +430,7 @@ Defaults to C<API CHANGES>.
 =head2 minor
 
 Comma-delimited list of categories of changes considered minor.
-Defaults to C<ENHANCEMENTS>.
+Defaults to C<ENHANCEMENTS> and C<UNGROUPED>.
 
 =head2 revision
 
